from argument_esa_model.esa import ESA
from conf.configuration import *
import csv
esa_model_debatepedia = None
esa_model_strategic_intelligence = None
import pandas as pd
import argument_esa_model.esa_all_terms


def load_topics(topic_ontology):
        path_topics = get_path_topics(topic_ontology)
        topics_df=pd.read_csv(path_topics,sep=",",quoting=csv.QUOTE_ALL,quotechar='"',encoding='utf-8').sort_values('name')
        return list(topics_df['name'])

def load_topics_with_ids(topic_ontology):
    path_topics = get_path_topics(topic_ontology)
    topics_df=pd.read_csv(path_topics,sep=",",quoting=csv.QUOTE_ALL,quotechar='"',encoding='utf-8',dtype={'id': str}).sort_values('name')
    return list(topics_df['name']),list(topics_df['id'])


#save_topics_for('ontology-strategic-intelligence-sub-topics')

#topics=load_topics('ontology-wikipedia')
#print(topics)
#esa_model_debatepedia,esa_model_strategic_intelligence=initialize_models()
#save_topics_for('ontology-strategic-intelligence')
#save_topics_for('ontology-debatepedia')


import logging
from datetime import datetime
import random
def setup_logging(file):
    logging.basicConfig(format="%(message)s",filename=file,level=logging.DEBUG)
    logging.warning(datetime.now())
def log(message):
    logging.warning(message)

def log_status(counter,size,debug):
    percentage=(float(counter)/size)*100
    if debug and random.randint(0,100) %13 ==0:
        logging.warning("finished %2.2f"%percentage)

def log_matches(id,topic_id,debug):
    if debug and random.randint(0,100) %13 ==0:
        logging.warning("matched ontology topic id %s for %d"%(topic_id,id))
def log_size(dataset,size):
    logging.warning("size of %s is %d"%(dataset,size))


import pandas as pd
import matplotlib.pyplot as plt
from docutils.nodes import thead

from conf.configuration import *
import numpy as np
import csv
from topic_modeling.topics import *
import sys

def get_top_topics_for(dataset, topic_ontology, topic_model, threshold=None):
    granularity = get_granularity(dataset)
    if threshold!=None:

        if granularity =='document':
            path_top_k_topic_path = get_path_topics_per_document_threshold(dataset,topic_ontology,topic_model,threshold)
        else:
            path_top_k_topic_path = get_path_topics_per_argument_threshold(dataset,topic_ontology,topic_model,threshold)

        df_topics = pd.read_csv(path_top_k_topic_path,quotechar='"',sep=",",quoting=csv.QUOTE_ALL,encoding="utf-8",dtype={'topic.id':object})
    else:
        if granularity =='document':
            path_top_k_topic_path = get_path_top_k_topics_per_document(dataset,topic_ontology,topic_model,1)
        else:
            path_top_k_topic_path = get_path_top_k_topics_per_argument(dataset,topic_ontology,topic_model,1)
        df_topics = pd.read_csv(path_top_k_topic_path,quotechar='"',sep=",",quoting=csv.QUOTE_ALL,encoding="utf-8",dtype={'topic.id':object})

    #TODO possible bug of counting documents. Probabliy document ids are not unique
    df_topic_per_document= df_topics.groupby('topic.id').agg({'document.id':pd.Series.nunique}).rename(columns={'document.id':"document.count"}).reset_index()




    topics =list(df_topic_per_document['topic.id'])
    documents_count=list(df_topic_per_document['document.count'])
    return {topic_id:documents_count[index]for index,topic_id in enumerate(topics)}

def get_top_topics_for_all_corpora(topic_ontology,topic_model,threshold=None):

    corpora = load_corpora_list()

    corpora_entries=[]
    topics=[]
    document_counts=[]

    ontology_topics,ontology_topic_ids= load_topics_with_ids('ontology-'+topic_ontology)
    for corpus in corpora:
        print(corpus)
        try:
            topics_with_document_count= get_top_topics_for(corpus, topic_ontology, topic_model, threshold)
            for topic_id in ontology_topic_ids:
                topics.append(topic_id)
                if topic_id in topics_with_document_count:
                    document_counts.append(topics_with_document_count[topic_id])
                else:
                    document_counts.append(0.0)
                corpora_entries.append(corpus)

        except Exception as error:
            print( error)
            print("%s Nope!" %corpus)

    return corpora_entries,topics,document_counts


def save_top_topics_with_document_counts(topic_ontology, topic_model,threshold=None):
    if threshold!=None:

        path_histogram = get_path_histogram_two_parameters('ontology-'+topic_ontology,'document-count','%s-%s-threshold'%(topic_ontology,topic_model))
        corpora,topic_ids,document_counts= get_top_topics_for_all_corpora(topic_ontology,topic_model,threshold)
    else:
        path_histogram = get_path_histogram_two_parameters('ontology-'+topic_ontology,'document-count','%s-%s'%(topic_ontology,topic_model))
        corpora,topic_ids,document_counts= get_top_topics_for_all_corpora(topic_ontology,topic_model)

    df_corpora=pd.DataFrame({'corpus':corpora,"topic.id":topic_ids,"document.count":document_counts})
    df_corpora.to_csv(path_histogram,encoding='utf-8',sep=',')

def load_top_topics_with_document_counts(topic_ontology, topic_model,threshold=None):
    if threshold!=None:
        path_histogram = get_path_histogram_two_parameters('ontology-'+topic_ontology,'document-count','%s-%s-threshold'%(topic_ontology,topic_model))
        df_top_topics_with_document_counts = pd.read_csv(path_histogram,encoding='utf-8',sep=',',dtype={'topic.id':str})
    else:
        path_histogram = get_path_histogram_two_parameters('ontology-'+topic_ontology,'document-count','%s-%s'%(topic_ontology,topic_model))
        df_top_topics_with_document_counts = pd.read_csv(path_histogram,encoding='utf-8',sep=',',dtype={'topic.id':str})
    return df_top_topics_with_document_counts

def load_topic_map(ontology):
    topic_map={}
    topics,topic_ids= load_topics_with_ids('ontology-'+ontology)

    for topic_id,topic in zip(topic_ids,topics):
        topic_map[str(topic_id)]=topic
    return topic_map

def visualize_top_topics_bar_diagram(topic_ontology, topic_model,threshold=None):
    df_top_topics_with_document_counts= load_top_topics_with_document_counts(topic_ontology,topic_model,threshold)
    df_top_topics_with_document_counts=df_top_topics_with_document_counts.groupby('topic.id').agg({'document.count':sum}).reset_index()
    topic_map=load_topic_map(topic_ontology)
    df_top_topics_with_document_counts=df_top_topics_with_document_counts.sort_values('document.count',ascending=False)
    #df_top_topics_with_document_counts=df_top_topics_with_document_counts[df_top_topics_with_document_counts['document.count']>100 ]
    topics = [topic_map[topic_id] for topic_id in list(df_top_topics_with_document_counts['topic.id'])]
    document_counts = df_top_topics_with_document_counts['document.count'].astype(int).values
    if threshold!=None:
        path_histogram = get_path_histogram_two_parameters('ontology-'+topic_ontology,'document-count','%s-%s-threshold-agg-by-topic'%(topic_ontology,topic_model))
    else:
        path_histogram = get_path_histogram_two_parameters('ontology-'+topic_ontology,'document-count','%s-%s-agg-by-topic'%(topic_ontology,topic_model))
    print(topics)
    print(document_counts)
    df_documents_over_topics_all_corpora=pd.DataFrame({'topic.id':df_top_topics_with_document_counts['topic.id'],'topic':topics,'document.count':document_counts})
    df_documents_over_topics_all_corpora.to_csv(path_histogram,sep=",",encoding='utf-8')
    if threshold!=None:
        path_figure_histogram = get_path_figure_histogram_two_parameters('ontology-' + topic_ontology, 'document-count', '%s-%s-threshold-agg-by-topic' % (topic_ontology, topic_model))
    else:
        path_figure_histogram = get_path_figure_histogram_two_parameters('ontology-' + topic_ontology, 'document-count', '%s-%s-agg-by-topic' % (topic_ontology, topic_model))
    fig, ax = plt.subplots(figsize=(120, 20))
    plt.xticks(rotation=90)
    plt.bar(topics,document_counts)
    fig.savefig(path_figure_histogram)

def visualize_top_topics_plot(topic_ontology, topic_model,threshold):
    df_top_topics_with_document_counts= load_top_topics_with_document_counts(topic_ontology,topic_model,threshold)
    df_top_topics_with_document_counts=df_top_topics_with_document_counts.groupby('topic.id').agg({'document.count':sum}).reset_index()
    topic_map=load_topic_map(topic_ontology)
    df_top_topics_with_document_counts=df_top_topics_with_document_counts.sort_values('document.count',ascending=False)
    #df_top_topics_with_document_counts=df_top_topics_with_document_counts[df_top_topics_with_document_counts['document.count']>100 ]
    topics = [topic_map[topic_id] for topic_id in list(df_top_topics_with_document_counts['topic.id'])]
    document_counts = df_top_topics_with_document_counts['document.count'].astype(int).values
    if threshold!=None:
        path_histogram = get_path_histogram_two_parameters('ontology-'+topic_ontology,'document-count','%s-%s-threshold-agg-by-topic'%(topic_ontology,topic_model))
    else:
        path_histogram = get_path_histogram_two_parameters('ontology-'+topic_ontology,'document-count','%s-%s-agg-by-topic'%(topic_ontology,topic_model))
    print(topics)
    print(document_counts)
    df_documents_over_topics_all_corpora=pd.DataFrame({'topic.id':df_top_topics_with_document_counts['topic.id'],'topic':topics,'document.count':document_counts})
    df_documents_over_topics_all_corpora.to_csv(path_histogram,sep=",",encoding='utf-8')
    if threshold!=None:
        path_figure_plot = get_path_figure_plot_two_parameters('ontology-' + topic_ontology, 'document-count', '%s-%s-threshold-agg-by-topic' % (topic_ontology, topic_model))
    else:
        path_figure_plot = get_path_figure_plot_two_parameters('ontology-' + topic_ontology, 'document-count', '%s-%s-agg-by-topic' % (topic_ontology, topic_model))


    fig, ax = plt.subplots(figsize=(40, 20))
    plt.xticks(rotation=90)
    plt.plot(topics,document_counts)
    fig.savefig(path_figure_plot)


def visualize_interactive_heat_map_threshold(topic_ontology, topic_model, threshold):
    #Dataframe <-read.csv("/mnt/ceph/storage/data-in-progress/topic-ontologies/topic-ontologies/ontoglies/wikipedia/document-over-wikipedia-esa-threshold-hisotgram.csvspread")
    # rownames(dataframe) <- dataframe$corpus
    #heatmaply(dataframe,color = c("white","black"),file="/home/yamenajjour/overleaf/coling20-topic-ontologies-for-argumentation/coling20-topic-ontologies-for-argumentation-data/heatmap.html")
    if threshold!=None:
        path_histogram = get_path_histogram_two_parameters('ontology-'+topic_ontology,'document-count','%s-%s-threshold'%(topic_ontology,topic_model))
    else:
        path_histogram = get_path_histogram_two_parameters('ontology-'+topic_ontology,'document-count','%s-%s'%(topic_ontology,topic_model))

    df = pd.read_csv(path_histogram,encoding='utf-8',sep=',',dtype={'topic.id':object})
    def load_topic_map(ontology):
        topic_map={}
        topics,topic_ids= load_topics_with_ids('ontology-'+ontology)

        for topic_id,topic in zip(topic_ids,topics):
            topic_map[str(topic_id)]=topic
        return topic_map
    df = df.pivot(index='corpus',columns='topic.id',values='document.count')
    df=df.reset_index()
    topic_map=load_topic_map('debatepedia')
    print(df.info())
    df=df.set_index('corpus')
    df.rename(columns=topic_map,inplace=True)
    df_sum=df.sum(axis=1)
    df_normalized=df.div(df_sum,axis=0)
    df_normalized=df_normalized.mul(100)
    df_normalized.to_csv(path_histogram+"spread",sep=",",encoding="utf-8")

#save_top_topics_with_document_counts('debatepedia', 'esa')
#save_top_topics_with_document_counts('strategic-intelligence', 'esa')
#save_top_topics_with_document_counts('wikipedia', 'esa')

#visualize_top_topics_bar_diagram('strategic-intelligence','esa')
#visualize_top_topics_bar_diagram('debatepedia','esa')
#visualize_top_topics_bar_diagram('wikipedia','esa')

ontology = sys.argv[1]
model = sys.argv[2]
threshold= float(sys.argv[3])

save_top_topics_with_document_counts(ontology, model,threshold)
visualize_top_topics_bar_diagram(ontology,model,threshold)
visualize_top_topics_plot(ontology,model,threshold)




from topic_modeling.topics import *
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from tqdm import tqdm
def load_corpus_vectors(dataset,topic_ontology,topic_model,is_argument_corpus):

    if is_argument_corpus:
        path_topic_vectors = get_path_argument_vectors(dataset,topic_ontology,topic_model)
    else:
        path_topic_vectors = get_path_document_vectors(dataset,topic_ontology,topic_model)
    vectors_df = pd.read_pickle(path_topic_vectors)
    if is_argument_corpus:
        corpora_vectors = list(vectors_df['argument-vector'])
    else:
        corpora_vectors = list(vectors_df['document-vector'])
    sum=np.zeros(len(corpora_vectors[0]))
    for vector in corpora_vectors:
        sum = sum + vector
    vector_norm = np.sum(sum)
    normalized_vector = sum/vector_norm
    return normalized_vector

def get_top_topics(corpus_vector,num):
    top_ten_threshold = sorted(corpus_vector,)[-num]
    filtered_topic_vector = []
    for topic_weight in corpus_vector:
        if topic_weight< top_ten_threshold:
            filtered_topic_vector.append(0)
        else:
            filtered_topic_vector.append(topic_weight)
    return filtered_topic_vector

def load_corpora_with_vectors_for_visualization(topic_ontology,topic_model):
    argument_corpora= ['args-me','ibm-debater-evidence-sentences','ukp-ukpconvarg-v1','ukp-ukpconvarg-v2','ibm-debater-evidence-quality']
    corpora = load_corpora_list()
    corpora_vectors= []
    for corpus in tqdm(corpora):
        print(corpus)
        if corpus in argument_corpora:
            corpora_vectors.append(load_corpus_vectors(corpus,topic_ontology,topic_model,True))
        else:
            corpora_vectors.append(load_corpus_vectors(corpus,topic_ontology,topic_model,False))

    corpora_vectors_np = np.array(corpora_vectors)
    return corpora,corpora_vectors_np

def load_corpora_with_vectors(topic_ontology,topic_model):
    argument_corpora= ['args-me','ibm-debater-evidence-sentences','ukp-ukpconvarg-v1','ukp-ukpconvarg-v2','ibm-debater-evidence-quality']
    corpora = load_corpora_list()
    topics = load_topics('ontology-'+topic_ontology)
    corpora_vectors= []
    corpora_topic_weights = {}
    for corpus in tqdm.tqdm(corpora):
        if corpus in argument_corpora:
            corpora_vectors.append(load_corpus_vectors(corpus,topic_ontology,topic_model,True))
        else:
            corpora_vectors.append(load_corpus_vectors(corpus,topic_ontology,topic_model,False))

    filtered_corpora_vectors = [get_top_topics(corpus_vector,20) for corpus_vector in corpora_vectors]
    for topic_index,topic in enumerate(topics):
        corpora_topic_weights[topic]=[]
        for corpus_index,corpus in enumerate(corpora):
            corpora_topic_weights[topic].append(filtered_corpora_vectors[corpus_index][topic_index])
    corpora_topic_weights['corpus']=corpora
    return corpora,corpora_topic_weights


def save_corpora_vectors(topic_ontology,topic_model):
    path_histogram = get_path_histogram_two_parameters('ontology-'+topic_ontology,'document-count','%s-%s'%(topic_ontology,topic_model))
    corpora,filtered_corpora_vectors= load_corpora_with_vectors(topic_ontology,topic_model)
    df_corpora=pd.DataFrame(filtered_corpora_vectors)
    df_corpora.to_csv(path_histogram,encoding='utf-8',sep=',')


def visualize_horizontal_bar_chart(topic_ontology,topic_model):
    path_horizontal_path_histograms = get_path_figure_histogram_two_parameters('ontology-' + topic_ontology, 'document-count', '%s-%s' % (topic_ontology, topic_model))

    topics = load_topics('ontology-'+topic_ontology)
    corpora,corpora_vectors= load_corpora_with_vectors_for_visualization(topic_ontology,topic_model)

    fig, ax = plt.subplots(figsize=(40, 20))
    ax.set_xticks(np.arange(len(topics)))
    ax.set_yticks(np.arange(len(corpora)))
    # ... and label them with the respective list entries
    ax.set_xticklabels(topics)
    ax.set_yticklabels(corpora)

    # Rotate the tick labels and set their alignment.
    plt.setp(ax.get_xticklabels(), rotation=45, ha="right",
             rotation_mode="anchor")


    im = ax.imshow(corpora_vectors,cmap="YlGn",)
    fig.tight_layout()
    fig.savefig(path_horizontal_path_histograms)



visualize_horizontal_bar_chart('strategic-intelligence','esa')


import pandas as pd
from conf.configuration import *
import csv
from sklearn.metrics import f1_score,recall_score,precision_score
import logging
from datetime import datetime
import random
from numpy import mean
from mylogging.mylogging import *
def produce_judgements(experiment,debug=False):
    dataset=get_path_source_part(experiment,'dataset')
    ontology=get_path_source_part(experiment,'ontology')
    path_document_topics=get_path_document_topic_ontology(dataset,ontology)
    path_judgements=get_path_judgement(experiment)
    df_document_topics=pd.read_csv(path_document_topics,sep=",",encoding="utf-8",quotechar='"',dtype={'ontology-topic-id':str})
    df_document_topics=df_document_topics[~df_document_topics['ontology-topic-id'].isin(['-1','-2'])]
    df_document_topics.to_csv(path_judgements,sep=",",encoding="utf-8",quotechar='"',index=False)





def load_labels(df_document_topics):
    df_document_topics.sort_values('id',inplace=True)
    df_document_topics_agg=df_document_topics.groupby('id').agg({'ontology-topic-id':list}).reset_index()

    document_ids = df_document_topics_agg['id'].values
    topic_ids = df_document_topics_agg['ontology-topic-id'].values

    document_with_topics=list(zip(document_ids,topic_ids))

    return document_with_topics

def load_judgements(experiment):
    path_judgements=get_path_judgement(experiment)
    df_document_topics=pd.read_csv(path_judgements,sep=",",encoding="utf-8",quotechar='"',dtype={'ontology-topic-id':str})
    labels = load_labels(df_document_topics)
    return labels


def drop_documents_not_in_judgements(df_suggestions, documents_with_topics):
    df_suggestions=df_suggestions[df_suggestions['id'].isin(documents_with_topics)]
    return df_suggestions


def load_predicted_topics_top_k(dataset,ontology,model,k,documents_with_topics):

    path_top_k_results=get_path_top_k_topics_per_document(dataset,ontology,model,k)
    df_suggestions=pd.read_csv(path_top_k_results,sep=",",encoding="utf-8",quotechar='"',dtype={'ontology-topic-id':str})
    df_suggestions=drop_documents_not_in_judgements(df_suggestions,documents_with_topics)
    labels =load_labels(df_suggestions)
    return labels
def load_ontology_topics(ontology):
    path_ontology_topics=get_path_topics('ontology-'+ontology)
    df_ontology_topics=pd.read_csv(path_ontology_topics,sep=',',quotechar='"',encoding='utf-8',dtype={'id': object})
    return df_ontology_topics['id'].values


def load_predicted_topics_threshold(dataset, ontology, model, threshold, documents_with_topics):
    path_top_k_results=get_path_topics_per_document_threshold(dataset,ontology,model,threshold)
    df_suggestions=pd.read_csv(path_top_k_results,sep=",",encoding="utf-8",quotechar='"',dtype={'ontology-topic-id':str})
    df_suggestions=drop_documents_not_in_judgements(df_suggestions,documents_with_topics)
    labels =load_labels(df_suggestions)
    return labels


def run_experiment(experiment,model,configuration,configuration_value,debug):
    dataset=get_path_source_part(experiment,'dataset')
    ontology=get_path_source_part(experiment,'ontology')
    ontology_topics=load_ontology_topics(ontology)
    documents_with_true_topics=load_judgements(experiment)
    documents_with_topics=[document_topics[0] for document_topics in documents_with_true_topics]
    if configuration=="top-k":
        documents_with_predicted_topics=load_predicted_topics_top_k(dataset,ontology,model,configuration_value,documents_with_topics)
    else:
        documents_with_predicted_topics=load_predicted_topics_threshold(dataset,ontology,model,configuration_value,documents_with_topics)
    effectiveness=evaluate(documents_with_true_topics,documents_with_predicted_topics,ontology_topics,debug)
    return effectiveness

def evaluate_threshold(experiment,model,threshold,debug=False):
    dataset=get_path_source_part(experiment,'dataset')
    ontology=get_path_source_part(experiment,'ontology')
    ontology_topics=load_ontology_topics(ontology)
    documents_with_true_topics=load_judgements(experiment)
    documents_with_topics=[document_topics[0] for document_topics in documents_with_true_topics]
    documents_with_predicted_topics=load_predicted_topics_top_k(dataset,ontology,model,k,documents_with_topics)
    effectiveness=evaluate(documents_with_true_topics,documents_with_predicted_topics,ontology_topics,debug)
    return effectiveness

def generate_labels(relevant_topics,ontology_topics):
    labels=[]
    for topic in ontology_topics:
        if topic in relevant_topics:
            labels.append(True)
        else:
            labels.append(False)
    return labels

def evaluate_sklearn(relevant_topics, predicted_topics, ontology_topics,debug=False):
    all_relevant_precisions=[]
    all_relevant_recalls=[]
    all_relevant_f1s=[]

    all_unrelevant_precisions=[]
    all_unrelevant_recalls=[]
    all_unrelevant_f1s=[]

    all_precisions=[]
    all_recalls=[]
    all_f1s=[]

    for i,document_topics in enumerate(relevant_topics):

        true_labels=generate_labels(document_topics[1],ontology_topics)
        predicted_labels=generate_labels(predicted_topics[i][1],ontology_topics)
        f1=f1_score(true_labels,predicted_labels,average='macro')
        recall=recall_score(true_labels,predicted_labels,average='macro')
        precision=precision_score(true_labels,predicted_labels,average='macro')
        relevant_f1= f1_score(true_labels,predicted_labels,average="binary",pos_label=True)
        relevant_recall=recall_score(true_labels,predicted_labels,average="binary",pos_label=True)
        relevant_precision=precision_score(true_labels,predicted_labels,average="binary",pos_label=True)

        unrelevant_f1= f1_score(true_labels,predicted_labels,average="binary",pos_label=False)
        unrelevant_recall=recall_score(true_labels,predicted_labels,average="binary",pos_label=False)
        unrelevant_precision=precision_score(true_labels,predicted_labels,average="binary",pos_label=False)

        all_recalls.append(recall)
        all_precisions.append(precision)
        all_f1s.append(f1)

        all_relevant_precisions.append(relevant_precision)
        all_relevant_recalls.append(relevant_recall)
        all_relevant_f1s.append(relevant_f1)

        all_unrelevant_precisions.append(unrelevant_precision)
        all_unrelevant_recalls.append(unrelevant_recall)
        all_unrelevant_f1s.append(unrelevant_f1)

    overall_f1=mean(all_f1s)
    overall_recall=mean(all_recalls)
    overall_precision=mean(all_precisions)

    overall_relevant_f1=mean(all_relevant_f1s)
    overall_relevant_precision=mean(all_relevant_precisions)
    overall_relevant_recall=mean(all_relevant_recalls)

    overall_unrelevant_f1=mean(all_unrelevant_f1s)
    overall_unrelevant_precision=mean(all_unrelevant_precisions)
    overall_unrelevant_recall=mean(all_unrelevant_recalls)

    return {'f1':overall_f1,'recall':overall_recall,'precision':overall_precision,
            'relevant-f1':overall_relevant_f1,'relevant-recall':overall_relevant_recall,'relevant-precision':overall_relevant_precision,
            'unrelevant-recall':overall_unrelevant_recall,'unrelevant-precision':overall_unrelevant_precision,'unrelevant-f1':overall_unrelevant_f1
            }

def evaluate(relevant_topics, predicted_topics, ontology_topics,debug=False):
    #true_labels=[(1,[1,2,3,4]),(2,[4,5,6,3]),...,]
    #predicted_labels=[(1,[
    ontology_topics_set=set(ontology_topics)

    all_relevant_precisions=[]
    all_relevant_recalls=[]
    all_relevant_f1s=[]

    all_unrelevant_precisions=[]
    all_unrelevant_recalls=[]
    all_unrelevant_f1s=[]

    all_precisions=[]
    all_recalls=[]
    all_f1s=[]
    size=len(list(relevant_topics))

    for i,document_topics in enumerate(relevant_topics):
        log_status(i,size,debug)
        relevant_topics_set=set(document_topics[1])
        unrelevant_topics_set=ontology_topics_set.difference(relevant_topics_set)
        predicted_topics_set=set(predicted_topics[i][1])
        unpredicted_topics_set=ontology_topics_set.difference(predicted_topics_set)
        if document_topics[0]!=predicted_topics[i][0]:
            raise ValueError("mismtach")

        true_positives = relevant_topics_set.intersection(predicted_topics_set)
        true_negatives = unrelevant_topics_set.intersection(unpredicted_topics_set)
        if len(predicted_topics_set)==0:
            relevant_precision=0
        else:
            relevant_precision= len(true_positives)/float(len(predicted_topics_set))
        if len(relevant_topics_set)==0:
            relevant_recall=0
        else:
            relevant_recall= len(true_positives)/float(len(relevant_topics_set))

        if relevant_recall==0 and relevant_precision ==0:
            relevant_f1=0
        else:
            relevant_f1=2*relevant_precision*relevant_recall/(relevant_precision+relevant_recall)


        unrelevant_precision=len(true_negatives)/float(len(unpredicted_topics_set))
        unrelevant_recall=len(true_negatives)/float(len(unrelevant_topics_set))
        unrelevant_f1=2*unrelevant_precision*unrelevant_recall/(unrelevant_precision+unrelevant_recall)

        f1=(relevant_f1+unrelevant_f1)/2.0
        recall=(relevant_recall+unrelevant_recall)/2.0
        precision=(relevant_precision+unrelevant_precision)/2.0

        all_recalls.append(recall)
        all_precisions.append(precision)
        all_f1s.append(f1)

        all_relevant_precisions.append(relevant_precision)
        all_relevant_recalls.append(relevant_recall)
        all_relevant_f1s.append(relevant_f1)

        all_unrelevant_precisions.append(unrelevant_precision)
        all_unrelevant_recalls.append(unrelevant_recall)
        all_unrelevant_f1s.append(unrelevant_f1)

    overall_f1=mean(all_f1s)
    overall_recall=mean(all_recalls)
    overall_precision=mean(all_precisions)

    overall_relevant_f1=mean(all_relevant_f1s)
    overall_relevant_precision=mean(all_relevant_precisions)
    overall_relevant_recall=mean(all_relevant_recalls)

    overall_unrelevant_f1=mean(all_unrelevant_f1s)
    overall_unrelevant_precision=mean(all_unrelevant_precisions)
    overall_unrelevant_recall=mean(all_unrelevant_recalls)

    return {'f1':overall_f1,'recall':overall_recall,'precision':overall_precision,
            'relevant-f1':overall_relevant_f1,'relevant-recall':overall_relevant_recall,'relevant-precision':overall_relevant_precision,
            'unrelevant-recall':overall_unrelevant_recall,'unrelevant-precision':overall_unrelevant_precision,'unrelevant-f1':overall_unrelevant_f1
            }

def run_experiments(experiment,model,k_s,thresholds,debug=False):
    # code logging
    path_evalaution=get_path_evaluation_model(experiment,model)
    precisions=[]
    recalls=[]
    f1_scores=[]
    true_precisions=[]
    true_recalls=[]
    true_f1_scores=[]
    false_precisions=[]
    false_recalls=[]
    false_f1_scores=[]
    for k in k_s:
        log("running %s for top-%d"%(experiment,k))
        effectiveness=run_experiment(experiment,model,'top-k',k,debug)
        precisions.append(effectiveness['precision'])
        recalls.append(effectiveness['recall'])
        f1_scores.append(effectiveness['f1'])
        true_precisions.append(effectiveness['relevant-precision'])
        true_recalls.append(effectiveness['relevant-recall'])
        true_f1_scores.append(effectiveness['relevant-f1'])
        false_recalls.append(effectiveness['unrelevant-recall'])
        false_precisions.append(effectiveness['unrelevant-precision'])
        false_f1_scores.append(effectiveness['unrelevant-f1'])
    for threshold in thresholds:
        log("running %s for threshold %2.2f}"%(experiment,threshold))
        effectiveness=evaluate_threshold(experiment,model,'threshold',threshold,debug)
        precisions.append(effectiveness['precision'])
        recalls.append(effectiveness['recall'])
        f1_scores.append(effectiveness['f1'])
        true_precisions.append(effectiveness['relevant-precision'])
        true_recalls.append(effectiveness['relevant-recall'])
        true_f1_scores.append(effectiveness['relevant-f1'])
        false_recalls.append(effectiveness['unrelevant-recall'])
        false_precisions.append(effectiveness['unrelevant-precision'])
        false_f1_scores.append(effectiveness['unrelevant-f1'])

    configurations= ["top-k-%d"%k for k in k_s]
    configurations.extend(["threshold-%f"%threshold for threshold in thresholds])

    df_evaluation=pd.DataFrame({'configuration':configurations,'precision':precisions,'recall':recalls,'f1':f1_scores,'relevant-precision':true_precisions,'relevant-recall':true_recalls,'relevant-f1':true_f1_scores
                                ,'unrelevant-precision':false_precisions,'unrelevant-recall':false_recalls,'unrelevant-f1':false_f1_scores
                                })
    df_evaluation.to_csv(path_evalaution,sep=",",encoding="utf-8",columns=['k','precision','recall','f1','relevant-precision','relevant-recall','relevant-f1',
                                                                           'unrelevant-precision','unrelevant-recall','unrelevant-f1'
                                                                           ],index=False)

